# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictStr, conint
except ImportError:
    from pydantic import BaseModel, Field, StrictStr, conint


class ListeningModeRef(BaseModel):
    """
    Reference to a listening mode  # noqa: E501
    """

    href: Optional[StrictStr] = Field(
        default=...,
        description="Host relative URI of the listening mode or `null` if there is no active listening mode. ",
    )
    id: Optional[conint(strict=True, ge=0)] = Field(
        default=...,
        description="ID of the active listening mode or `null` if there is no active listening mode. ",
    )
    __properties = ["href", "id"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ListeningModeRef:
        """Create an instance of ListeningModeRef from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict["href"] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict["id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ListeningModeRef:
        """Create an instance of ListeningModeRef from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ListeningModeRef.parse_obj(obj)

        _obj = ListeningModeRef.parse_obj(
            {"href": obj.get("href"), "id": obj.get("id")}
        )
        return _obj
