# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, StrictInt
except ImportError:
    from pydantic import BaseModel, Field, StrictInt


class PowerLinkTrigger(BaseModel):
    """
    PowerLinkTrigger
    """

    power_link_preset: Optional[StrictInt] = Field(
        default=None,
        alias="powerLinkPreset",
        description="Despite the name, this preset is used anytime a speaker is used as a secondary as a way for the primary to indicate which directivity (and possibly other caracteristics in the future) should be used on a speaker by speaker basis. It is used when a speaker is a powerlink speaker (hence the name), WISA speaker or a speakerlink secondary in multichannel. ",
    )
    __properties = ["powerLinkPreset"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PowerLinkTrigger:
        """Create an instance of PowerLinkTrigger from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PowerLinkTrigger:
        """Create an instance of PowerLinkTrigger from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PowerLinkTrigger.parse_obj(obj)

        _obj = PowerLinkTrigger.parse_obj(
            {"power_link_preset": obj.get("powerLinkPreset")}
        )
        return _obj
