# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import io
import re  # noqa: F401
import warnings

try:
    from pydantic.v1 import ValidationError, validate_arguments
except ImportError:
    from pydantic import ValidationError, validate_arguments

from typing import Awaitable, Optional, Union, overload

from typing_extensions import Annotated

try:
    from pydantic.v1 import Field, StrictBool
except ImportError:
    from pydantic import Field, StrictBool


from typing import Optional

from mozart_api.api_client import ApiClient
from mozart_api.api_response import ApiResponse
from mozart_api.exceptions import ApiTypeError, ApiValueError  # noqa: F401
from mozart_api.models.stand_movement import StandMovement


class StandApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    async def set_stand_movement(
        self,
        stand_movement: Annotated[
            StandMovement, Field(..., description="Stand Movement")
        ],
        slow_speed: Annotated[
            Optional[StrictBool],
            Field(description="Use slow speed on continuous movement"),
        ] = None,
        **kwargs,
    ) -> None:  # noqa: E501
        """Set stand movement with either an angle or a movement type  # noqa: E501


        :param stand_movement: Stand Movement (required)
        :type stand_movement: StandMovement
        :param slow_speed: Use slow speed on continuous movement
        :type slow_speed: bool
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs["_return_http_data_only"] = True
        if "_preload_content" in kwargs:
            message = "Error! Please call the set_stand_movement_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return await self.set_stand_movement_with_http_info(
            stand_movement, slow_speed, **kwargs
        )  # noqa: E501

    @validate_arguments
    async def set_stand_movement_with_http_info(
        self,
        stand_movement: Annotated[
            StandMovement, Field(..., description="Stand Movement")
        ],
        slow_speed: Annotated[
            Optional[StrictBool],
            Field(description="Use slow speed on continuous movement"),
        ] = None,
        **kwargs,
    ) -> ApiResponse:  # noqa: E501
        """Set stand movement with either an angle or a movement type  # noqa: E501


        :param stand_movement: Stand Movement (required)
        :type stand_movement: StandMovement
        :param slow_speed: Use slow speed on continuous movement
        :type slow_speed: bool
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = ["stand_movement", "slow_speed"]
        _all_params.extend(
            [
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method set_stand_movement" % _key
                )
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get("slow_speed") is not None:  # noqa: E501
            _query_params.append(("slow-speed", _params["slow_speed"]))

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params["stand_movement"] is not None:
            _body_params = _params["stand_movement"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json"]
        )  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type",
            self.api_client.select_header_content_type(["application/json"]),
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {}

        return await self.api_client.call_api(
            "/api/v1/stand/move",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
        )
