# coding: utf-8

"""
Mozart platform API

API for interacting with the Mozart platform.

The version of the OpenAPI document: 0.2.0
Contact: support@bang-olufsen.dk
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field, conint
except ImportError:
    from pydantic import BaseModel, Field, conint

from mozart_api.models.video_pixel_format import VideoPixelFormat
from mozart_api.models.video_timings import VideoTimings


class HdmiVideoFormat(BaseModel):
    """
    HdmiVideoFormat
    """

    pixel_format: Optional[VideoPixelFormat] = Field(default=None, alias="pixelFormat")
    v_ic: Optional[conint(strict=True, ge=0)] = Field(default=None, alias="vIC")
    video_timings: Optional[VideoTimings] = Field(default=None, alias="videoTimings")
    __properties = ["pixelFormat", "vIC", "videoTimings"]

    class Config:
        """Pydantic configuration"""

        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> HdmiVideoFormat:
        """Create an instance of HdmiVideoFormat from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True, exclude={}, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of pixel_format
        if self.pixel_format:
            _dict["pixelFormat"] = self.pixel_format.to_dict()
        # override the default output from pydantic by calling `to_dict()` of video_timings
        if self.video_timings:
            _dict["videoTimings"] = self.video_timings.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> HdmiVideoFormat:
        """Create an instance of HdmiVideoFormat from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return HdmiVideoFormat.parse_obj(obj)

        _obj = HdmiVideoFormat.parse_obj(
            {
                "pixel_format": VideoPixelFormat.from_dict(obj.get("pixelFormat"))
                if obj.get("pixelFormat") is not None
                else None,
                "v_ic": obj.get("vIC"),
                "video_timings": VideoTimings.from_dict(obj.get("videoTimings"))
                if obj.get("videoTimings") is not None
                else None,
            }
        )
        return _obj
